#include "mbed.h"
#include "EthernetNetIf.h"
#include "HTTPServer.h"
#include "HTTPClient.h"
#include "NTPClient.h"
#include "SerialRPCInterface.h"
//  OTHER INCLUDES ...

DigitalOut led1(LED1, "led1");
DigitalOut led2(LED2, "led2");
DigitalOut led3(LED3, "led3");
DigitalOut led4(LED4, "led4");

// IP Settings
IpAddr myIP(192,168,1,128);
IpAddr netmask(255,255,255,0);
IpAddr gateway(192,168,1,1);
IpAddr defdns(192,168,1,1);
// file system
LocalFileSystem fs("webfs");


//Time handling
time_t ctTime;
bool isTimeSet=false;

NTPClient ntp;
Host ntpserver(IpAddr(), 123, "0.uk.pool.ntp.org");
NTPResult NTPres;

// Serial port
void Rxhandler(void);
Serial stat_serial(NC, p27);

// OTHER VARIABLES...

// HTTP client
HTTPClient MyClient;
void HttpGetCallback(HTTPResult);

char url[128];
char reply[1024];

// IP interface
EthernetNetIf eth(myIP,netmask,gateway,defdns);  
HTTPServer svr;


int main() {
  
  //HTTP Client
  MyClient.setOnResult(HttpGetCallback);
  MyClient.setTimeout(5000);
      
  printf("Setting up...\n");
  
  // Serial port
  stat_serial.attach(Rxhandler,Serial::RxIrq);
  
  
  // Ethernet, IP interface
  EthernetErr ethErr = eth.setup();
  if(ethErr)
  {
    printf("Error %d in setup.\n", ethErr);
    return -1;
  }
  printf("Setup OK\n");
  
  //Setting time, based on network (NTP request)
   ctTime = time(NULL);  
  printf("Time before setup is (UTC): %s\n", ctime(&ctTime));
  // Loop for retrying
  while(1)
  {
      
      NTPres=ntp.setTime(ntpserver);
      if(NTPres==NTP_OK)
      {
        isTimeSet=true;
        ctTime = time(NULL);
        printf("NTP query success, time is now (UTC): %s\n", ctime(&ctTime));
        break;  // Exiting from loop
      }
      else
      {
        isTimeSet=false;
        printf("NTP query failed, retry...\n");
      }
  
  }
  
  // File system
  FSHandler::mount("/webfs", "/files"); //Mount /webfs path on /files web path
  FSHandler::mount("/webfs", "/"); //Mount /webfs path on web root path
  
  // HTTP Server
  svr.addHandler<RPCHandler>("/rpc");
  svr.addHandler<FSHandler>("/files");
  svr.addHandler<FSHandler>("/"); //Default handler
  //Example : Access to mbed.htm : http://a.b.c.d/mbed.htm or http://a.b.c.d/files/mbed.htm
  
  //Start listening
  svr.bind(80);
  printf("Listening...\n");
    
  // Timer for LED blinking (showing that we're alive)
  Timer tm;
  tm.start();
  //Listen indefinitely
  
  while(true)
  {
    Net::poll(); // What is this line doing? It is necessary for the server, that's clear
    
    //Handling an UART based event...
    if (msg_flag)
    {
				// Some file writing...
               
                MyClient.doGet(url, NULL);
             
        
    msg_flag=false;
    }
    
    // For blinking LED:
    if(tm.read()>0.5)
    {
      led1=!led1; //Show that we are alive
      tm.start();
  
    }
  }
  
  return 0;

}

void Rxhandler(void)
{
    led3=1;
    // Some character copying...
    msg_flag=true;

}

void HttpGetCallback(HTTPResult)
{
    // I don't need the answer for the request
    led4=!led4;
}
